@@ -23,20 +23,20 @@ import (
 	networking "k8s.io/api/networking/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/auth"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/authz"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/connection"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/locationmodifier"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/parser"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/proxy"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/rewrite"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/secureupstream"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/snippet"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/upstreamhashby"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/upstreamuri"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/xforwardedprefix"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/errors"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/resolver"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/auth"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/authz"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/connection"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/locationmodifier"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/parser"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/proxy"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/rewrite"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/secureupstream"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/snippet"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/upstreamhashby"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/upstreamuri"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/xforwardedprefix"
+	"github.com/stolostron/management-ingress/pkg/ingress/errors"
+	"github.com/stolostron/management-ingress/pkg/ingress/resolver"
 )
 
 // DeniedKeyName name of the key that contains the reason to deny a location
