@@ -35,19 +35,19 @@ import (
 	"k8s.io/client-go/tools/record"
 	"k8s.io/client-go/util/flowcontrol"
 
-	"github.com/open-cluster-management/management-ingress/pkg/file"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/annotations/class"
-	ngx_config "github.com/open-cluster-management/management-ingress/pkg/ingress/controller/config"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/controller/process"
-	ngx_template "github.com/open-cluster-management/management-ingress/pkg/ingress/controller/template"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/status"
-	"github.com/open-cluster-management/management-ingress/pkg/ingress/store"
-	ing_net "github.com/open-cluster-management/management-ingress/pkg/net"
-	"github.com/open-cluster-management/management-ingress/pkg/net/dns"
-	"github.com/open-cluster-management/management-ingress/pkg/task"
-	"github.com/open-cluster-management/management-ingress/pkg/watch"
+	"github.com/stolostron/management-ingress/pkg/file"
+	"github.com/stolostron/management-ingress/pkg/ingress"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations"
+	"github.com/stolostron/management-ingress/pkg/ingress/annotations/class"
+	ngx_config "github.com/stolostron/management-ingress/pkg/ingress/controller/config"
+	"github.com/stolostron/management-ingress/pkg/ingress/controller/process"
+	ngx_template "github.com/stolostron/management-ingress/pkg/ingress/controller/template"
+	"github.com/stolostron/management-ingress/pkg/ingress/status"
+	"github.com/stolostron/management-ingress/pkg/ingress/store"
+	ing_net "github.com/stolostron/management-ingress/pkg/net"
+	"github.com/stolostron/management-ingress/pkg/net/dns"
+	"github.com/stolostron/management-ingress/pkg/task"
+	"github.com/stolostron/management-ingress/pkg/watch"
 )
 
 var (
